/*-------------------------------------------------------------------------------------------------------------+
|                                                       					|
|   Trigger: CTRL_FILIAL                     					|
|   Tabela: Filial		           					|
|                                                       					|
|   Descrio:							|
|   Atualiza os dados da HISTFILIAL quando os campos QTFUNCFILI,	|
|   CDGOPERAC OU CDSUPERVISOR so alterados.			|
|								|
|   Criao:                                       					|
|   18/01/2008 - Luiz Passos               					|
|                                                       					|
+-------------------------------------------------------------------------------------------------------------*/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
	WHERE
		o.name = 'CTRL_FILIAL'		and
		type = 'TR'                   		and
		u.uid = o.uid                		and
		u.name = USER)
		DROP TRIGGER CTRL_FILIAL
GO

CREATE TRIGGER CTRL_FILIAL
ON FILIAL
FOR INSERT, UPDATE, DELETE
AS
	DECLARE	@V_ID_COMANDO		VARCHAR(1);
	DECLARE	@V_DTINCLUSAO			DATETIME;

	DECLARE	@V_MES_OLD			VARCHAR(2);
	DECLARE	@V_ANO_OLD			VARCHAR(4);
	DECLARE	@V_CDFILIAL_OLD		VARCHAR(4);
	DECLARE	@V_QTFUNCFILI_OLD		NUMERIC;
	DECLARE	@V_CDGOPERAC_OLD		VARCHAR(12);
	DECLARE	@V_CDSUPERVISOR_OLD		VARCHAR(12);

	DECLARE	@V_MES_NEW			VARCHAR(2);
	DECLARE	@V_ANO_NEW			VARCHAR(4);
	DECLARE	@V_CDFILIAL_NEW		VARCHAR(4);
	DECLARE	@V_QTFUNCFILI_NEW		NUMERIC;
	DECLARE	@V_CDGOPERAC_NEW		VARCHAR(12);
	DECLARE	@V_CDSUPERVISOR_NEW	VARCHAR(12);

BEGIN      

	DECLARE C_FILIAL CURSOR FOR

		SELECT
			'D' IDCOMANDO,
			CONVERT(VARCHAR,MONTH(GETDATE())),
			CONVERT(VARCHAR,YEAR(GETDATE())),
			CDFILIAL,
			QTFUNCFILI,
			CDGOPERAC,
			CDSUPERVISOR,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL
			FROM
			DELETED
			WHERE
			CDFILIAL NOT IN (SELECT CDFILIAL FROM INSERTED)

			UNION ALL

		SELECT
			'I' IDCOMANDO,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			CONVERT(VARCHAR,MONTH(GETDATE())),
			CONVERT(VARCHAR,YEAR(GETDATE())),
			CDFILIAL,
			QTFUNCFILI,
			CDGOPERAC,
			CDSUPERVISOR
		FROM
			INSERTED
		WHERE
			CDFILIAL NOT IN (SELECT CDFILIAL FROM DELETED)

		UNION ALL

		SELECT
			'U' IDCOMANDO,
			CONVERT(VARCHAR,MONTH(GETDATE())),
			CONVERT(VARCHAR,YEAR(GETDATE())),
			DELETED.CDFILIAL,
			DELETED.QTFUNCFILI,
			DELETED.CDGOPERAC,
			DELETED.CDSUPERVISOR,
			CONVERT(VARCHAR,MONTH(GETDATE())),
			CONVERT(VARCHAR,YEAR(GETDATE())),
			INSERTED.CDFILIAL,
			INSERTED.QTFUNCFILI,
			INSERTED.CDGOPERAC,
			INSERTED.CDSUPERVISOR
		FROM
			INSERTED,
			DELETED
		WHERE
			INSERTED.CDFILIAL = DELETED.CDFILIAL AND
			(ISNULL(INSERTED.QTFUNCFILI,0)   <> ISNULL(DELETED.QTFUNCFILI,0) OR
			ISNULL(INSERTED.CDGOPERAC,'')    <> ISNULL(DELETED.CDGOPERAC,'')  OR	
			ISNULL(INSERTED.CDSUPERVISOR,'') <> ISNULL(DELETED.CDSUPERVISOR,''));


	OPEN C_FILIAL;
	FETCH NEXT FROM C_FILIAL INTO 
		@V_ID_COMANDO,
		@V_MES_OLD,
		@V_ANO_OLD,
		@V_CDFILIAL_OLD,
		@V_QTFUNCFILI_OLD,
		@V_CDGOPERAC_OLD,
		@V_CDSUPERVISOR_OLD,

		@V_MES_NEW,
		@V_ANO_NEW,
		@V_CDFILIAL_NEW,
		@V_QTFUNCFILI_NEW,
		@V_CDGOPERAC_NEW,
		@V_CDSUPERVISOR_NEW;

	WHILE (@@FETCH_STATUS = 0) BEGIN

		SET @V_DTINCLUSAO = CONVERT(DATETIME,('01/'+@V_MES_NEW+'/'+@V_ANO_NEW) ,103);

		IF @V_ID_COMANDO = 'I' BEGIN

			INSERT INTO HISTFILIAL
				(CDFILIAL, 	
				DTREFEHIST,
				QTFUNCFILIHIST, 
				CDGOPERACHIST,
				CDSUPERVHIST)
			VALUES
				(@V_CDFILIAL_NEW,
 				 @V_DTINCLUSAO,
				 @V_QTFUNCFILI_NEW,
				 @V_CDGOPERAC_NEW,
				 @V_CDSUPERVISOR_NEW);

		END
		ELSE IF @V_ID_COMANDO = 'U' BEGIN

			IF (0 = (SELECT COUNT(CDFILIAL) FROM HISTFILIAL 
				WHERE CDFILIAL = @V_CDFILIAL_NEW AND DTREFEHIST = @V_DTINCLUSAO)) BEGIN
			
				INSERT INTO HISTFILIAL
					(CDFILIAL, 	
					DTREFEHIST,
					QTFUNCFILIHIST, 
					CDGOPERACHIST,
					CDSUPERVHIST)
				VALUES
					(@V_CDFILIAL_NEW,
					@V_DTINCLUSAO,
					@V_QTFUNCFILI_NEW,
					@V_CDGOPERAC_NEW,
					@V_CDSUPERVISOR_NEW);
			END
			ELSE IF (ISNULL(@V_QTFUNCFILI_NEW,0)    <> ISNULL(@V_QTFUNCFILI_OLD,0))    OR 
				(ISNULL(@V_CDGOPERAC_NEW,'')    <> ISNULL(@V_CDGOPERAC_OLD,''))    OR
				(ISNULL(@V_CDSUPERVISOR_NEW,'') <> ISNULL(@V_CDSUPERVISOR_OLD,'')) BEGIN

				UPDATE HISTFILIAL SET
					QTFUNCFILIHIST = @V_QTFUNCFILI_NEW,
					CDGOPERACHIST  = @V_CDGOPERAC_NEW,
					CDSUPERVHIST   = @V_CDSUPERVISOR_NEW
				WHERE
					CDFILIAL   = @V_CDFILIAL_OLD AND
					DTREFEHIST = @V_DTINCLUSAO;
			END;
		END
		ELSE IF @V_ID_COMANDO = 'D' BEGIN
			DELETE FROM HISTFILIAL WHERE CDFILIAL = @V_CDFILIAL_OLD;
		END;

		FETCH NEXT FROM C_FILIAL INTO 
			@V_ID_COMANDO,
			@V_MES_OLD,
			@V_ANO_OLD,
			@V_CDFILIAL_OLD,
			@V_QTFUNCFILI_OLD,
			@V_CDGOPERAC_OLD,
			@V_CDSUPERVISOR_OLD,
				
			@V_MES_NEW,
			@V_ANO_NEW,
			@V_CDFILIAL_NEW,
			@V_QTFUNCFILI_NEW,
			@V_CDGOPERAC_NEW,
			@V_CDSUPERVISOR_NEW;
	END;
	CLOSE C_FILIAL;
	DEALLOCATE C_FILIAL;
END
GO

-- VERSAO --
EXEC ATUVEROBJ 'CTRL_FILIAL','5.01.005';
GO
